# base::ipmi and base::noipmi
#
# ipmi classes to load, and suppress loading, ipmi client modules.
# Loading of ipmi is automatically suppressed on VMware platforms.

class base::ipmi {

  # disable ipmi and ipmievd on vmware (or others). facter string comparison
  if $::is_virtual == 'true' or $::ec2_profile == 'default-paravirtual' {
    case $::operatingsystem {
      'debian', 'ubuntu': {
        package { 'ipmitool': ensure => purged }
      }
      'redhat': {
        service { 'ipmi':
          ensure  => stopped,
          enable  => false,
        }
      }
      default: {}
    }
  } else {
    case $::osfamily {
      'Debian': {
        package { 'ipmitool':
          ensure => present;
        }
        if ($::kernelmajversion >= '3.13') {
          # no msghandler module on kernels > 3.13
          base::os::kernel_module { 'ipmi_si':
            ensure  => present,
          }
        }
        else {
          base::os::kernel_module { 'ipmi_msghandler':
            ensure => present;
          }
          base::os::kernel_module { 'ipmi_si':
            ensure  => present,
            require => Base::Os::Kernel_module['ipmi_msghandler'];
          }
        }
        base::os::kernel_module { 'ipmi_devintf':
          ensure  => present,
          require => Base::Os::Kernel_module['ipmi_si'];
        }
        service { 'ipmievd':
          ensure      => running,
          name        => 'ipmievd',
          enable      => true,
          hasstatus   => false,
          status      => 'pidof ipmievd',
          require     => [Base::Os::Kernel_module['ipmi_devintf'],
                          File['/etc/default/ipmievd']
                          ],
        }
        file { '/etc/default/ipmievd':
          content => template('base/ipmi/etc/default/ipmievd.erb'),
          notify  => Service['ipmievd'],
        }
      }
      'RedHat': {
        case $::lsbmajdistrelease {
          '6': {
            package { 'OpenIPMI':   ensure => present; }
          }
          default: {
            package {
              'OpenIPMI-tools':   ensure => present;
              'OpenIPMI':         ensure => present;
            }
          }
        }
        service { 'ipmi':
          ensure    => running,
          name      => 'ipmi',
          enable    => true,
          hasstatus => true,
          require   => Package['OpenIPMI'],
        }
        case $::lsbmajdistrelease {
          '5', '6': {
            service { 'ipmievd':
              ensure      => running,
              name        => 'ipmievd',
              enable      => true,
              hasstatus   => true,
              require     => $::lsbmajdistrelease ? {
                '5' => Package['OpenIPMI-tools'],
                default    => Package['OpenIPMI'],
              },
            }
          }
        }
      }
    }

    # Disable cipher zero.  Some IPMI report the ciphers backwards so we work
    # around it. Puppet 2.6.3 doesn't support provider param for exec.
    # Working around it.  The signal file /etc/noipmi allows this execution to
    # be suppressed on systems that don't have ipmi.
    exec { 'ipmitool lan set 1 cipher_privs XaaaaaaaaaaaaaX':
      unless => 'bash -c \'if [ ! -e /etc/noipmi ] && ! `ipmitool lan print | grep -q "Cipher Suite Priv Max   : Not Available"`; then ipmitool lan print | grep "Cipher Suite Priv Max" | cut -d: -f2 | egrep -q "^ XaaaaaaaaaaaaaX"; fi\'',
    }
  }
}

class base::noipmi inherits base::ipmi {
  Base::Os::Kernel_module['ipmi_si']      { ensure => absent }
  Base::Os::Kernel_module['ipmi_devintf'] { ensure => absent }
  File['/etc/default/ipmievd']            { ensure => absent }

  Service['ipmievd'] { ensure => stopped }

  file { '/etc/noipmi':
    mode    => '0644',
    content => "ipmi is not supported on this system\n",
  }
}
